const TAG_FIELDS = `id,name`;
const NODE_FIELDS = `id,label,icon,type,asset,slug,hideLabel,left,right,level,` +
    `rootNode,parentNode,channel,humanReadablePath,path,url,createdAt`;
const NODE_EXPAND = `asset,icon,asset.tags`;

class App
{
    constructor () {
        this.helper = new Helper();
        this.DOM = {};
    }

    init () {
        this.DOM.assetList = document.querySelector('.asset-list');
        this.DOM.title = document.querySelector('.title');

        this.setLoading(true);

        this.loadTags(tags => {
            if (tags === null) {
                this.helper.displayError('Could not load tags');
            }
            else if (tags.length < 1) {
                this.helper.displayError('No tags found');
            }
            else {
                this.findChannelNodesWithTags(
                    tags.map(tag => tag.id),
                    assets => this.display(tags.map(tag => tag.name), assets)
                );
            }
        })
    }

    setLoading (loading) {
        if (loading) {
            document.body.className += ' loading';
        }
        else {
            document.body.className = document.body.className
                .replace(/\bloading\b/g, '')
                .replace(/\b\s\s\b/, ' ')
                .trim();
        }
    }

    loadTags (callbackFn) {
        const endpoint = `tags.json?fields=${TAG_FIELDS}&limit=2`;

        window.ShowpadLib.getShowpadApi(apiConfig => onShowpadApiConfig.call(this, apiConfig));

        function onShowpadApiConfig (apiConfig) {
            if (apiConfig !== null) {
                this.helper.getJson(endpoint, apiConfig, callbackFn);
            }
            else {
                callbackFn(null);
            }
        }
    }

    findChannelNodesWithTags (tagIds, callbackFn) {
        const endpoint = `channelnodes.json?assetTagIds=${tagIds.join('%2C')}&fields=${NODE_FIELDS}&expand=${NODE_EXPAND}`;

        window.ShowpadLib.getShowpadApi(apiConfig => onShowpadApiConfig.call(this, apiConfig));

        function onShowpadApiConfig (apiConfig) {
            if (apiConfig !== null) {
                this.helper.getJson(endpoint, apiConfig, callbackFn);
            }
            else {
                callbackFn(null);
            }
        }
    }

    display (tags, nodes) {
        this.setLoading(false);

        if (!nodes) {
            this.helper.displayError('Could not load channel nodes');
        }
        else {
            this.DOM.title.innerHTML = `Asset with ${ tags.length > 1 ? 'tags' : 'tag' } ` +
                `${tags.map(tag => `<i>${tag}</i>`).join(' or ')}`;

            this.DOM.assetList.innerHTML = nodes
                .map(node => this.helper.nodeDataToHtml(node))
                .join('');
        }
    }
}

class Helper {
    displayError (message) {
        // simple alert
        alert(`Something went wrong\n\n${message}`);
    }

    getJson (endpoint, apiConfig, callbackFn) {
        const url = `${apiConfig.url}/api/v3/${endpoint}`;
        const config = {
            headers: {
                Authorization: `Bearer ${apiConfig.accessToken}`
            }
        };

        fetch(url, config)
            .then(response => {
                if (response.ok) {
                    return response
                        .json()
                        .then(data => callbackFn(data.response.items));
                }
                else {
                    callbackFn(null);
                }
            })
            .catch(() => callback(null));
    }

    nodeDataToHtml (node) {
        return `
            <li>
                <a href="${node.asset.appLink}?modal=true">
                    <div class="preview" style="background-image: url(${node.asset.previewDownloadLink})"></div>
                    <label>${node.label}</label>
                </a>
            </li>
        `;
    }
}


window.onShowpadLibLoaded = () => {
    const app = new App();
    app.init();
};
